% Roll Support and Camber Compensation
clear all, close all

% vehicle data
g  = 9.81 ;  % [m/s^2] constant of cravity
ay = 0.75*g; % [m/s^2] set lateral acceleration
mc = 600;    % [kg]    chassis mass
mw =  50;    % [kg]    mass of each knuckle and wheel
s  = 1.5;    % [m]     track width
h0 = 0.6;    % [m]     height of center of gravity
r0 = 0.3;    % [m]     static tire radius
cs = 20000;  % [N/m]   suspension stiffness
cb = 10000;  % [N/m]   stiffness of anti roll bar
cy =180000;  % [N/m]   tire lateral stiffness
cz =200000;  % [N/m]   tire radial stiffness

% axle kinematics
dydz = 0 ;   % [-] lateral motion of wheel center caused by vertical
dadz = 0/180*pi; % [Grad/m] camber change caused by vertical motion

hr = h0-r0 ; sh = s/2;  dyqdz = dydz + r0*dadz; % some abbreviations

% stiffness matrix (column by column)
K(:,1) = [        2*cy         ; ...
                   0           ; ...     
                2*cy*h0        ; ...
                cy*dyqdz       ; ...
               -cy*dyqdz       ];
K(:,2) = [         0           ; ...
                  2*cz         ; ...
                   0           ; ...
                   cz          ; ...
                   cz          ];
K(:,3) = [       2*cy*h0       ; ...
                    0          ; ...
           2*cz*sh^2+2*cy*h0^2 ; ...
            sh*cz+h0*dyqdz*cy  ; ...
           -sh*cz-h0*dyqdz*cy  ];
K(:,4) = [        cy*dyqdz     ; ...
                     cz        ; ...
             sh*cz+h0*dyqdz*cy ; ...
           cs+cb+cz+dyqdz^2*cy ; ...
                  -cb          ];
K(:,5) = [      -cy*dyqdz      ; ...
                    cz         ; ...
           -sh*cz-h0*dyqdz*cy  ; ...
                     -cb       ; ...
          cs+cb+cz+dyqdz^2*cy  ];

b = -[ mc+2*mw;  0;  2*mw*hr;  mw*dydz; -mw*dydz ]*ay;  % right hand side             

x = K\b; % solve set of linear equations and display results  
disp(['lateral displacement of chassis  y_C =',num2str(x(1))])
disp(['vertical displacement of chassis z_C =',num2str(x(2))])
disp(['roll motion of chassis       alpha_C =',num2str(x(3)*180/pi)])
disp(['suspension travel inner wheel    z_1 =',num2str(x(4))])
disp(['suspension travel outer wheel    z_2 =',num2str(x(5))])

% compute and display wheel loads
Fz1=(mw+mc/2)*g+cz*(x(2)+s/2*x(3)+x(4)); disp(['Fz1=',num2str(Fz1)])
Fz2=(mw+mc/2)*g+cz*(x(2)-s/2*x(3)+x(5)); disp(['Fz2=',num2str(Fz2)])


% animate results

% get states
ya = x(1); za=x(2); aa=x(3); z1=x(4); z2=x(5);

% Vehicle Body
ya0 = [ 1; 1; -1; -1; ] * 1.2*sh  ;
za0 = [-1; 1;  1; -1; ] * 0.25*(h0+r0) ;
yr0 = [ 1; 1; -1; -1; ] * 0.4*r0;
zr0 = [-1; 1;  1; -1; ] * r0;

% wheels
yr1 = yr0 + sh; zr1 = zr0 + r0;
yr2 = yr0 - sh; zr2 = zr0 + r0;

% Road
hf=figure;
set(hf,'units','normalized')
set(hf,'name',['Steady State Cornering at a_y/g = ',num2str(ay/g),' [-]'])

plot([-1.5; 1.5]*sh,[0; 0],'k','Linewidth',3), hold on

% Initial Configuration
plot([ya0;ya0(1)],[za0;za0(1)]+h0,'b')
plot([yr1;yr1(1)],[zr1;zr1(1)],'b')
plot([yr2;yr2(1)],[zr2;zr2(1)],'b')

% Steady State Cornering
yay = cos(aa).*ya0 - sin(aa).*za0;
zay = sin(aa).*ya0 + cos(aa).*za0;
plot([yay;yay(1)]+ya,[zay;zay(1)]+h0+za,'k','Linewidth',1.5)

al = aa+dadz*z1 ;
yrq = cos(al).*yr0 - sin(al).*zr0;
zrq = sin(al).*yr0 + cos(al).*zr0;
yrq = yrq +      ya + cos(aa)*(sh+dydz*z1) - sin(aa)*(-hr+z1) ;
zrq = zrq + h0 + za + sin(aa)*(sh+dydz*z1) + cos(aa)*(-hr+z1) ;
plot([yrq;yrq(1)],[zrq;zrq(1)],'k','Linewidth',1.5)

al = aa-dadz*z2;
yrq = cos(al).*yr0 - sin(al).*zr0;
zrq = sin(al).*yr0 + cos(al).*zr0;
yrq = yrq +      ya + cos(aa)*(-sh-dydz*z2) - sin(aa)*(-hr+z2) ;
zrq = zrq + h0 + za + sin(aa)*(-sh-dydz*z2) + cos(aa)*(-hr+z2) ;
plot([yrq;yrq(1)],[zrq;zrq(1)],'k','Linewidth',1.5)

% Wheel Center
ym1 =      ya + cos(aa)*(+sh+dydz*z1) - sin(aa)*(-hr+z1) ;
zm1 = h0 + za + sin(aa)*(+sh+dydz*z1) + cos(aa)*(-hr+z1) ;
ym2 =      ya + cos(aa)*(-sh-dydz*z2) - sin(aa)*(-hr+z2) ;
zm2 = h0 + za + sin(aa)*(-sh-dydz*z2) + cos(aa)*(-hr+z2) ;

% Contact Points
yp1 = ym1 + zm1*tan(aa+dadz*z1);
zp1 = 0;
yp2 = ym2 + zm2*tan(aa-dadz*z2);
zp2 = 0;

plot(yp1,zp1,'or','Linewidth',1.5)
plot(yp2,zp2,'or','Linewidth',1.5)

% Roll Center
dy=1.2*(yp2-yp1)/2;
plot([yp1,yp1+dy],[zp1;-dy*dyqdz],'r','Linewidth',0.5)
dy=1.2*(yp1-yp2)/2;
plot([yp2,yp2+dy],[zp2;+dy*dyqdz],'r','Linewidth',0.5)

% Some Text
ztmax = h0+max(za0); ztmin = h0+min(za0);
%
zt=ztmin + 0.75*(ztmax-ztmin);
h=text(-0.75*sh,zt,['y_A = ',num2str(ya*1000),' [mm]']);
set(h,'HorizontalAlignment','center')
h=text( 0.75*sh,zt,['\alpha_A = ',num2str(aa*180/pi),' [deg]']);
set(h,'HorizontalAlignment','center')
zt=ztmin + 0.25*(ztmax-ztmin);
h=text(-0.50*sh,zt,['z_2 = ',num2str(z2*1000),' [mm]']);
set(h,'HorizontalAlignment','center')
h=text(+0.50*sh,zt,['z_1 = ',num2str(z1*1000),' [mm]']);
set(h,'HorizontalAlignment','center')

axis equal, axis off

% Compression and Rebound

hf=figure; hold on
set(hf,'units','normalized')
set(hf,'name','Axle Kinematics')

for i=-5:5
  z = i/5*0.1;
  ym = sh + dydz*z;
  zm = r0 + z ;
  a = dadz*z;
  yrq =  ym  +  cos(a).*yr0 - sin(a).*zr0;
  zrq =  zm  +  sin(a).*yr0 + cos(a).*zr0;
  plot([yrq;yrq(1)],[zrq;zrq(1)],'b')
  plot((yrq(1)+yrq(4))/2,(zrq(1)+zrq(4))/2,'.b')
  plot([ym],[zm],'.r')
  ym = -sh - dydz*z;
  zm =  r0 + z ;
  a = -dadz*z;
  yrq = ym  +  cos(a).*yr0 - sin(a).*zr0;
  zrq = zm  +  sin(a).*yr0 + cos(a).*zr0;
  plot([yrq;yrq(1)],[zrq;zrq(1)],'b')
  plot((yrq(1)+yrq(4))/2,(zrq(1)+zrq(4))/2,'.b')
  plot([ym],[zm],'.r')
end

% Initial Configuration
plot([ya0;ya0(1)],[za0;za0(1)]+h0,'k','Linewidth',2), hold on
plot([yr1;yr1(1)],[zr1;zr1(1)],'k','Linewidth',1.5)
plot([yr2;yr2(1)],[zr2;zr2(1)],'k','Linewidth',1.5)

axis equal , axis off
