% double wishbone kinematics (rill, september 2010)
clear all, close all

% suspension kinematics (typical passenger car front left)
dt.umx  = 0.0745;  % max. rack displacement 
dt.phimx= 10/180*pi; % max rotation of lower control arm
dt.rvwk= [ 0.0000;  0.7680;  0.0000]; % W: wheel rim center
dt.rvak= [-0.2510;  0.3200; -0.0800]; % A: lower control arm @ chassis rear
dt.rvbk= [ 0.1480;  0.3200; -0.0940]; % B: lower control arm @ chassis front
dt.rvck= [ 0.0130;  0.7370; -0.1450]; % C: lower control arm @ wheel body
dt.rvdk= [-0.1050;  0.4350;  0.1960]; % D: upper control arm @ body rear 
dt.rvek= [ 0.1220;  0.4350;  0.2300]; % E: upper control arm @ body front
dt.rvfk= [-0.0250;  0.6800;  0.1620]; % F: upper control arm @ wheel body
dt.rvrk= [-0.1500;  0.3800; -0.0380]; % R: drag link @ rack
dt.rvqk= [-0.1370;  0.6900; -0.0880]; % Q: drag link @ wheel body 
% additional data 
toe0 = 0.0000/180*pi; % initial toe angle (sign according to ISO def)
camb0= 0.8000/180*pi; % initial camber angle (sign accord. to ISO def)
rs   = 0.2850; % steady state tire radius
en0  = [ 0; 0; 1 ]; % road normal (flat horizontal road)

% wheel/tire orientation in design position 
eyrk = [toe0; 1; -camb0];  eyrk = eyrk/norm(eyrk);  % wheel rot. axis
exk = cross(eyrk,en0); exk=exk/norm(exk);  % longitudinal direction
eyk = cross(en0,exk); % lateral direction
ezk = cross(exk,eyrk); % radial direction
rwpk= -rs*ezk; % wheel center W --> reference point P (contact in design pos.) 

% kingpin and caster angle in design position
rcfk = dt.rvfk-dt.rvck; ecfk=rcfk/norm(rcfk); % kingpin orientation
si = atan2(-ecfk(2),ecfk(3)); disp(['kingpin angle = ',num2str(si*180/pi)])
nu = atan2(-ecfk(1),ecfk(3)); disp(['caster angle = ',num2str(nu*180/pi)])  

% caster offset and scrub radius in design position
rcpk = dt.rvwk + rwpk - dt.rvck ; rsck = -(en0'*rcpk)/(en0'*ecfk)*ecfk ;
co = -exk'*(rsck+rcpk); disp(['caster offset = ',num2str(co)])
sr =  eyk'*(rsck+rcpk); disp(['scrub radius = ',num2str(sr)])

% range of motion (rotation of lower control arm and rack displacement)
n=11; phi=linspace(-1,1,n)*dt.phimx; m=15; u=linspace(-1,1,m)*dt.umx;

% pre-allocate vars to speed up loops
xw=zeros(n,m);yw=xw;zw=xw; xp=xw;yp=xw;zp=xw; del=xw; toe=xw; camb=xw; ddel=xw;
for i=1:n
  for j=1:m        
    [avw,rvwv,del(i,j),pd] = dblwb_kin(phi(i),u(j),dt); % suspension kinematics   
    eyrv = avw*eyrk; %  actual orientation of wheel rotation axis
    rvpv = rvwv + avw*rwpk; % actual position of reference point P
    xw(i,j)=rvwv(1);yw(i,j)=rvwv(2);zw(i,j)=rvwv(3); % wheel center coord.
    xp(i,j)=rvpv(1);yp(i,j)=rvpv(2);zp(i,j)=rvpv(3); % ref. point coord.
    toe(i,j) = atan2(-eyrv(1), eyrv(2)); % toe angle (+ rot. z-axis)
    camb(i,j)= atan2( eyrv(3), eyrv(2)); % camber angle (+ rot. x-axis)
    ddel(i,j)=norm(pd(:,2)); % partial derivative ddel/du
  end
end

n0=round(n/2); m0=round(m/2);  rvpk = dt.rvwk + rwpk ; 
axes('position',[0.05,0.05,0.20,0.90]),hold on,axis equal,grid on,title('xz')
plot(xw(:,m0),zw(:,m0)),plot(xp(:,m0),zp(:,m0),'--')
plot(dt.rvwk(1),dt.rvwk(3),'ok'), plot(rvpk(1),rvpk(3),'ok') 
axes('position',[0.30,0.05,0.20,0.90]),hold on,axis equal,grid on,title('yz')
plot(yw(:,m0),zw(:,m0)), plot(yp(:,m0),zp(:,m0),'--')
plot(dt.rvwk(2),dt.rvwk(3),'ok'),plot(rvpk(2),rvpk(3),'ok') 
axes('position',[0.60,0.05,0.35,0.40]), colormap('white')
surf(u,phi*180/pi,toe*180/pi), grid on, view(-40,10), title('toe')
axes('position',[0.60,0.55,0.35,0.40]), colormap('white')
surf(u,phi*180/pi,camb*180/pi), grid on, view(-40,10), title('camber')
figure % open new figure
d1 = del(n0,:); d2 = -d1(m:-1:1); % wheel steering angles in design position
a=2.7; s=2*dt.rvwk(2); d2a = atan2(a*tan(d1),a+s*tan(d1)); % ackermann angle
axes('position',[0.05,0.55,0.40,0.30]), title('\delta_1(u), \delta_2(u)')
plot(u,[d1;d2]*180/pi), grid on, legend('left','right')
axes('position',[0.05,0.15,0.40,0.30]), title('d\delta/du')
plot(u,ddel(n0,:)), grid on
axes('position',[0.55,0.30,0.40,0.40]), axis equal, title('\delta_2(\delta_1)')
plot(d1*180/pi,[d2;d2a]*180/pi),grid on, legend('kinematics','ackermann')


% 3d-plots
figure
aview=135; beview=60; colormap('white')
subplot(2,2,1), surf(u,phi*180/pi,xw-dt.rvwk(1)), view(aview,beview)
subplot(2,2,2), surf(u,phi*180/pi,yw-dt.rvwk(2)), view(aview,beview)
subplot(2,2,3), surf(u,phi*180/pi,zw-dt.rvwk(3)), view(aview,beview)
subplot(2,2,4), surf(u,phi*180/pi,del*180/pi), view(aview,beview)

figure
subplot(2,2,1), plot(phi,xw(:,m0))
subplot(2,2,2), plot(phi,yw(:,m0))
subplot(2,2,3), plot(phi,zw(:,m0))
subplot(2,2,4), plot(phi,del(:,m0))

figure
subplot(2,2,1), plot(u,xw(n0,:))
subplot(2,2,2), plot(u,yw(n0,:))
subplot(2,2,3), plot(u,zw(n0,:))
subplot(2,2,4), plot(u,del(n0,:))

